import type { Member } from '$lib/model/Member';
import jsPDF from 'jspdf';
import autoTable from 'jspdf-autotable';
import { printPlaceDate } from './common';

export function generateVotingCards({
	votingItems,
	members
}: {
	votingItems: string[];
	members: Member[];
}) {
	// const totalPagesExp = '{total_pages_count_string}';

	const voting = members.filter((x) => x.voting);

	const doc = new jsPDF({
		orientation: 'portrait',
		format: 'a6'
	});
	// eslint-disable-next-line @typescript-eslint/no-explicit-any
	(jsPDF as any).autoTableSetDefaults({
		bodyStyles: {
			lineColor: 0,
			lineWidth: 0.05
		},
		styles: {
			font: 'NotoSansLight'
		},
		columnStyles: {
			lineColor: [255, 255, 255],
			lineWidth: 1
		},
		headStyles: {
			font: 'NotoSansBold',
			fontSize: 7,
			textColor: 0,
			lineColor: 0,
			lineWidth: 0.05,
			fillColor: '#ddd'
		}
	});

	doc.addFont('/Noto_Sans/NotoSans-Light.ttf', 'NotoSansLight', 'normal');
	doc.addFont('/Noto_Sans/NotoSans-Regular.ttf', 'NotoSansRegular', 'normal');
	doc.addFont('/Noto_Sans/NotoSans-Bold.ttf', 'NotoSansBold', 'normal');

	doc.setFont('NotoSansRegular', 'normal'); // set font

	function makePage() {
		printPlaceDate(doc);

		doc.setFontSize(10);
		doc.setFont('NotoSansBold');
		doc.text('Karta do głosowania', 6, 7);

		const voteCell = { content: '', styles: { cellWidth: 15 } };
		const votingRows = votingItems.map((item) => [item, voteCell, voteCell, voteCell]);

		doc.setFont('NotoSansLight', 'normal'); // set font
		doc.setFontSize(5);

		autoTable(doc, {
			head: [['Kandydat / Sprawa', 'ZA', 'PRZECIW', 'WSTRZYMUJĘ SIĘ']],
			horizontalPageBreak: false,
			pageBreak: 'avoid',
			rowPageBreak: 'avoid',
			margin: { left: 6, right: 6 },
			body: [...votingRows].slice(0, 20)
		});

		doc.setFontSize(7);
		doc.setFont('NotoSansRegular', 'normal');
		doc.text(
			'Columns: Candidate/Matter | FOR | AGAINST | ABSTAIN',
			6,
			// eslint-disable-next-line @typescript-eslint/no-explicit-any
			(doc as any).lastAutoTable.finalY + 5
		);

		doc.setFont('NotoSansRegular', 'normal'); // set font

		doc.setFontSize(12);
		// doc.putTotalPages(totalPagesExp);
	}

	voting.forEach((_, i) => {
		if (i > 0) {
			doc.addPage();
		}
		makePage();
	});

	doc.save('voting-cards.pdf');
}
