import { redirect } from '@sveltejs/kit';
import { env } from '$env/dynamic/private';
import FakeHydrationService from '$lib/hydration/FakeHydrationService';
import type { IHydrationService } from '$lib/hydration/IHydrationService';
import { LdapHydrationService } from '$lib/hydration/LdapHydrationService.js';
import invariant from 'tiny-invariant';

/** @type {import('./$types').LayoutServerLoad} */
export async function load({ route, locals }) {
	const session = await locals.getSession();

	if (!session) {
		throw redirect(302, '/auth/signin');
	}

	let ldap = true;
	try {
		invariant(env.LDAP_USER_DN, 'LDAP_USER_DN is not defined, reverting to fake hydration');
		invariant(env.LDAP_USER_PW, 'LDAP_USER_PW is not defined, reverting to fake hydration');
	} catch (e) {
		ldap = false;
	}

	let hydrationService: IHydrationService;
	if (ldap) {
		hydrationService = new LdapHydrationService();
	} else {
		hydrationService = new FakeHydrationService();
	}

	const members = await hydrationService.hydrate();

	return {
		members
	};
}
